package cn.com.qmth.examcloud.core.oe.admin.service.impl;

import cn.com.qmth.examcloud.core.oe.admin.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.core.oe.admin.service.ExamStudentFinalScoreService;
import cn.com.qmth.examcloud.core.oe.admin.service.bean.SubjectiveQuestionScoreInfo;
import cn.com.qmth.examcloud.core.oe.common.base.utils.QuestionTypeUtil;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamQuestionEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordQuestionsEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamScoreEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.ExamType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordQuestionsRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamScoreRepo;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service("examRecordQuestionsService")
public class ExamRecordQuestionsServiceImpl implements ExamRecordQuestionsService {

    @Autowired
    private ExamRecordQuestionsRepo examRecordQuestionsRepo;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private ExamScoreRepo examScoreRepo;

    @Autowired
    private ExamStudentFinalScoreService examStudentFinalScoreService;

    @Override
    public List<ExamQuestionEntity> querySubjectiveAnswerList(Long examRecordDataId) {
        ExamRecordQuestionsEntity examRecordQuestionsEntity = examRecordQuestionsRepo.findByExamRecordDataId(examRecordDataId);
        List<ExamQuestionEntity> examRecordQuestionList = examRecordQuestionsEntity.getExamQuestionEntities();

        return examRecordQuestionList.stream().filter((obj -> {
            return !QuestionTypeUtil.isObjectiveQuestion(obj.getQuestionType());
        })).collect(Collectors.toList());
    }

    @Override
    public void saveSubjectiveQuestionScore(Long examRecordDataId, List<SubjectiveQuestionScoreInfo> subjectiveQuestionScoreInfoList) {
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId, ExamRecordDataEntity.class);
        if (examRecordData.getExamType() == ExamType.ONLINE) {
            ExamRecordQuestionsEntity examRecordQuestionsEntity = examRecordQuestionsRepo.findByExamRecordDataId(examRecordDataId);
            List<ExamQuestionEntity> examRecordQuestionList = examRecordQuestionsEntity.getExamQuestionEntities();

            for (SubjectiveQuestionScoreInfo scoreInfo : subjectiveQuestionScoreInfoList) {
                for (ExamQuestionEntity examQuestionEntity : examRecordQuestionList) {
                    if (scoreInfo.getQuestionId().equals(examQuestionEntity.getQuestionId())
                            && scoreInfo.getOrder().intValue() == examQuestionEntity.getOrder().intValue()) {
                        examQuestionEntity.setStudentScore(scoreInfo.getScore());
                        break;
                    }
                }
            }
            examRecordQuestionsRepo.save(examRecordQuestionsEntity);
        }
        //计算主观题总得分
        double totalSubjectiveScore = 0D;
        for (SubjectiveQuestionScoreInfo scoreInfo : subjectiveQuestionScoreInfoList) {
            totalSubjectiveScore += scoreInfo.getScore();
        }

        //更新ec_oe_exam_score
        ExamScoreEntity examScoreEntity = examScoreRepo.findByExamRecordDataId(examRecordDataId);
        examScoreEntity.setSubjectiveScore(totalSubjectiveScore);
        double totalScore = totalSubjectiveScore + examScoreEntity.getObjectiveScore();
        examScoreEntity.setTotalScore(totalScore);
        examScoreRepo.save(examScoreEntity);

        //计算考生的最终分数
        examStudentFinalScoreService.calculateFinalScore(examRecordData.getExamStudentId());
    }

}
